import 'dart:convert';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/config/settings.dart';
import 'package:scrobbles/cubit/data_timeline_cubit.dart';
import 'package:scrobbles/models/timeline.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/timeline_counts.dart';
import 'package:scrobbles/ui/widgets/charts/timeline_eclecticism.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardTimeline extends StatelessWidget {
  const CardTimeline({super.key});

  @override
  Widget build(BuildContext context) {
    final int daysCount = Settings.timelineDaysCount;

    return BlocBuilder<DataTimelineCubit, DataTimelineState>(
      builder: (BuildContext context, DataTimelineState state) {
        return CardContent(
          color: Theme.of(context).colorScheme.surface,
          title: 'timeline_title'.tr(
            namedArgs: {
              'daysCount': daysCount.toString(),
            },
          ),
          loader: updateTimeline(Settings.timelineDaysCount),
          content: Stack(
            children: [
              ChartTimelineCounts(
                chartData: TimelineData.fromJson(jsonDecode(state.timeline.toString())),
                isLoading: false,
              ),
              ChartTimelineEclecticism(
                chartData: TimelineData.fromJson(jsonDecode(state.timeline.toString())),
                isLoading: false,
              ),
            ],
          ),
        );
      },
    );
  }

  Widget updateTimeline(int daysCount) {
    final Widget loading = const Text('⏳');
    final Widget done = const Text('');

    late Future<TimelineData> futureTimeline = ScrobblesApi.fetchTimeline(daysCount);

    return BlocBuilder<DataTimelineCubit, DataTimelineState>(
      builder: (BuildContext context, DataTimelineState dataTimelineState) {
        return FutureBuilder<TimelineData>(
          future: futureTimeline,
          builder: (context, snapshot) {
            if (snapshot.hasError) {
              return ShowErrorWidget(message: '${snapshot.error}');
            }

              BlocProvider.of<DataTimelineCubit>(context).update(snapshot.data);

            return !snapshot.hasData ? loading : done;
          },
        );
      },
    );
  }
}
